@@ -4,13 +4,11 @@ use warnings;
 use autodie;
 package CPAN::Meta::Validator;
 BEGIN {
-  $CPAN::Meta::Validator::VERSION = '2.101461';
+  $CPAN::Meta::Validator::VERSION = '2.101580';
 }
 # ABSTRACT: validate CPAN distribution metadata structures
 
 
-use Carp qw(confess);
-
 #--------------------------------------------------------------------------#
 # This code copied and adapted from Test::CPAN::Meta
 # by Barbie, <barbie@cpan.org> for Miss Barbell Productions,
@@ -34,11 +32,21 @@ my $module_map1 = { 'map' => { ':key' => { name => \&module, value => \&exversio
 
 my $module_map2 = { 'map' => { ':key' => { name => \&module, value => \&version   } } };
 
+my $no_index_2 = {
+    'map'       => { file       => { list => { value => \&string } },
+                     directory  => { list => { value => \&string } },
+                     'package'  => { list => { value => \&string } },
+                     namespace  => { list => { value => \&string } },
+                    ':key'      => { name => \&custom_2, value => \&anything },
+    }
+};
+
 my $no_index_1_3 = {
     'map'       => { file       => { list => { value => \&string } },
                      directory  => { list => { value => \&string } },
                      'package'  => { list => { value => \&string } },
                      namespace  => { list => { value => \&string } },
+                     ':key'     => { name => \&string, value => \&anything },
     }
 };
 
@@ -47,6 +55,7 @@ my $no_index_1_2 = {
                      dir        => { list => { value => \&string } },
                      'package'  => { list => { value => \&string } },
                      namespace  => { list => { value => \&string } },
+                     ':key'     => { name => \&string, value => \&anything },
     }
 };
 
@@ -81,7 +90,8 @@ my %definitions = (
       mandatory => 1,
       'map' => {
         version => { mandatory => 1, value => \&version},
-        url     => { value => \&url }
+        url     => { value => \&url },
+        ':key' => { name => \&custom_2, value => \&anything },
       }
     },
     'name'                => { mandatory => 1, value => \&string  },
@@ -91,14 +101,15 @@ my %definitions = (
     # OPTIONAL
     'description' => { value => \&string },
     'keywords'    => { lazylist => { value => \&string } },
-    'no_index'    => $no_index_1_3,
+    'no_index'    => $no_index_2,
     'optional_features'   => {
       'map'       => {
         ':key'  => {
-          name => \&identifier,
+          name => \&string,
           'map'   => {
             description        => { value => \&string },
             prereqs => $prereq_map,
+            ':key' => { name => \&custom_2, value => \&anything },
           }
         }
       }
@@ -110,7 +121,8 @@ my %definitions = (
           name  => \&module,
           'map' => {
             file    => { mandatory => 1, value => \&file },
-            version => { value => \&version }
+            version => { value => \&version },
+            ':key' => { name => \&custom_2, value => \&anything },
           }
         }
       }
@@ -123,6 +135,7 @@ my %definitions = (
           'map' => {
             web => { value => \&url },
             mailto => { value => \&string},
+            ':key' => { name => \&custom_2, value => \&anything },
           }
         },
         repository => {
@@ -130,6 +143,7 @@ my %definitions = (
             web => { value => \&url },
             url => { value => \&url },
             type => { value => \&string },
+            ':key' => { name => \&custom_2, value => \&anything },
           }
         },
         ':key'     => { value => \&string, name => \&custom_2 },
@@ -142,8 +156,14 @@ my %definitions = (
   },
 
 '1.4' => {
-  'meta-spec'           => { mandatory => 1, 'map' => { version => { mandatory => 1, value => \&version},
-                                                        url     => { mandatory => 1, value => \&urlspec } } },
+  'meta-spec'           => {
+    mandatory => 1,
+    'map' => {
+      version => { mandatory => 1, value => \&version},
+      url     => { mandatory => 1, value => \&urlspec },
+      ':key'  => { name => \&string, value => \&anything },
+    },
+  },
 
   'name'                => { mandatory => 1, value => \&string  },
   'version'             => { mandatory => 1, value => \&version },
@@ -163,21 +183,28 @@ my %definitions = (
 
   'optional_features'   => {
     'map'       => {
-        ':key'  => { name => \&identifier,
+        ':key'  => { name => \&string,
             'map'   => { description        => { value => \&string },
                          requires           => $module_map1,
                          recommends         => $module_map1,
                          build_requires     => $module_map1,
                          conflicts          => $module_map2,
+                         ':key'  => { name => \&string, value => \&anything },
             }
         }
      }
   },
 
   'provides'    => {
-    'map'       => { ':key' => { name  => \&module,
-                                 'map' => { file    => { mandatory => 1, value => \&file },
-                                            version => { value => \&version } } } }
+    'map'       => {
+      ':key' => { name  => \&module,
+        'map' => {
+          file    => { mandatory => 1, value => \&file },
+          version => { value => \&version },
+          ':key'  => { name => \&string, value => \&anything },
+        }
+      }
+    }
   },
 
   'no_index'    => $no_index_1_3,
@@ -200,8 +227,14 @@ my %definitions = (
 },
 
 '1.3' => {
-  'meta-spec'           => { mandatory => 1, 'map' => { version => { mandatory => 1, value => \&version},
-                                                        url     => { mandatory => 1, value => \&urlspec } } },
+  'meta-spec'           => {
+    mandatory => 1,
+    'map' => {
+      version => { mandatory => 1, value => \&version},
+      url     => { mandatory => 1, value => \&urlspec },
+      ':key'  => { name => \&string, value => \&anything },
+    },
+  },
 
   'name'                => { mandatory => 1, value => \&string  },
   'version'             => { mandatory => 1, value => \&version },
@@ -220,23 +253,31 @@ my %definitions = (
 
   'optional_features'   => {
     'map'       => {
-        ':key'  => { name => \&identifier,
+        ':key'  => { name => \&string,
             'map'   => { description        => { value => \&string },
                          requires           => $module_map1,
                          recommends         => $module_map1,
                          build_requires     => $module_map1,
                          conflicts          => $module_map2,
+                         ':key'  => { name => \&string, value => \&anything },
             }
         }
      }
   },
 
   'provides'    => {
-    'map'       => { ':key' => { name  => \&module,
-                                 'map' => { file    => { mandatory => 1, value => \&file },
-                                            version => { value => \&version } } } }
+    'map'       => {
+      ':key' => { name  => \&module,
+        'map' => {
+          file    => { mandatory => 1, value => \&file },
+          version => { value => \&version },
+          ':key'  => { name => \&string, value => \&anything },
+        }
+      }
+    }
   },
 
+
   'no_index'    => $no_index_1_3,
   'private'     => $no_index_1_3,
 
@@ -261,8 +302,15 @@ my %definitions = (
 # original mistake, and that a v1.1 was retro fitted into the timeline, when
 # v1.2 was originally slated as v1.1. But I could be wrong ;)
 '1.2' => {
-  'meta-spec'           => { mandatory => 1, 'map' => { version => { mandatory => 1, value => \&version},
-                                                        url     => { mandatory => 1, value => \&urlspec } } },
+  'meta-spec'           => {
+    mandatory => 1,
+    'map' => {
+      version => { mandatory => 1, value => \&version},
+      url     => { mandatory => 1, value => \&urlspec },
+      ':key'  => { name => \&string, value => \&anything },
+    },
+  },
+
 
   'name'                => { mandatory => 1, value => \&string  },
   'version'             => { mandatory => 1, value => \&version },
@@ -286,21 +334,28 @@ my %definitions = (
 
   'optional_features'   => {
     'map'       => {
-        ':key'  => { name => \&identifier,
+        ':key'  => { name => \&string,
             'map'   => { description        => { value => \&string },
                          requires           => $module_map1,
                          recommends         => $module_map1,
                          build_requires     => $module_map1,
                          conflicts          => $module_map2,
+                         ':key'  => { name => \&string, value => \&anything },
             }
         }
      }
   },
 
   'provides'    => {
-    'map'       => { ':key' => { name  => \&module,
-                                 'map' => { file    => { mandatory => 1, value => \&file },
-                                            version => { value => \&version } } } }
+    'map'       => {
+      ':key' => { name  => \&module,
+        'map' => {
+          file    => { mandatory => 1, value => \&file },
+          version => { value => \&version },
+          ':key'  => { name => \&string, value => \&anything },
+        }
+      }
+    }
   },
 
   'resources'   => {
@@ -697,7 +752,7 @@ sub custom_1 {
     if(defined $key) {
         # a valid user defined key should be alphabetic
         # and contain at least one capital case letter.
-        return 1    if($key && $key =~ /^[a-z]+$/i && $key =~ /[A-Z]/);
+        return 1    if($key && $key =~ /^[_a-z]+$/i && $key =~ /[A-Z]/);
     } else {
         $key = '<undef>';
     }
@@ -708,13 +763,11 @@ sub custom_1 {
 sub custom_2 {
     my ($self,$key) = @_;
     if(defined $key) {
-        # a valid user defined key should be alphabetic
-        # and begin with x_ or X_
-        return 1    if($key && $key =~ /^x_([-_a-z]+)$/i);  # user defined
+        return 1    if($key && $key =~ /^x_/i);  # user defined
     } else {
         $key = '<undef>';
     }
-    $self->_error( "Custom resource '$key' must begin with 'x_' or 'X_'." );
+    $self->_error( "Custom key '$key' must begin with 'x_' or 'X_'." );
     return 0;
 }
 
@@ -745,6 +798,7 @@ sub phase {
     my ($self,$key) = @_;
     if(defined $key) {
         return 1 if( length $key && grep { $key eq $_ } @valid_phases );
+        return 1 if $key =~ /x_/i;
     } else {
         $key = '<undef>';
     }
@@ -757,6 +811,7 @@ sub relation {
     my ($self,$key) = @_;
     if(defined $key) {
         return 1 if( length $key && grep { $key eq $_ } @valid_relations );
+        return 1 if $key =~ /x_/i;
     } else {
         $key = '<undef>';
     }
@@ -786,7 +841,7 @@ CPAN::Meta::Validator - validate CPAN distribution metadata structures
 
 =head1 VERSION
 
-version 2.101461
+version 2.101580
 
 =head1 SYNOPSIS
 
